@@ -25,15 +25,16 @@ class Test(TestCase):
     @skipIf(version_info < (3,),
             'in Python 2 exception names stay bound after the except: block')
     def test_undefinedExceptionName(self):
-        """Exception names can't be used after the except: block."""
+        """Exception names can't be used after the except: block.
+
+        The exc variable is unused inside the exception handler."""
         self.flakes('''
         try:
             raise ValueError('ve')
         except ValueError as exc:
             pass
         exc
-        ''',
-                    m.UndefinedName)
+        ''', m.UndefinedName, m.UnusedVariable)
 
     def test_namesDeclaredInExceptBlocks(self):
         """Locals declared in except: blocks can be used after the block.
@@ -71,7 +72,8 @@ class Test(TestCase):
         """Exception names are unbound after the `except:` block.
 
         Last line will raise UnboundLocalError on Python 3 but would print out
-        've' on Python 2."""
+        've' on Python 2. The exc variable is unused inside the exception
+        handler."""
         self.flakes('''
         try:
             raise ValueError('ve')
@@ -79,14 +81,15 @@ class Test(TestCase):
             pass
         print(exc)
         exc = 'Original value'
-        ''',
-                    m.UndefinedName)
+        ''', m.UndefinedName, m.UnusedVariable)
 
     def test_undefinedExceptionNameObscuringLocalVariableFalsePositive1(self):
         """Exception names obscure locals, can't be used after. Unless.
 
         Last line will never raise UnboundLocalError because it's only
         entered if no exception was raised."""
+        # The exc variable is unused inside the exception handler.
+        expected = [] if version_info < (3,) else [m.UnusedVariable]
         self.flakes('''
         exc = 'Original value'
         try:
@@ -95,7 +98,7 @@ class Test(TestCase):
             print('exception logged')
             raise
         exc
-        ''')
+        ''', *expected)
 
     def test_delExceptionInExcept(self):
         """The exception name can be deleted in the except: block."""
@@ -111,6 +114,8 @@ class Test(TestCase):
 
         Last line will never raise UnboundLocalError because `error` is
         only falsy if the `except:` block has not been entered."""
+        # The exc variable is unused inside the exception handler.
+        expected = [] if version_info < (3,) else [m.UnusedVariable]
         self.flakes('''
         exc = 'Original value'
         error = None
@@ -122,7 +127,7 @@ class Test(TestCase):
             print(error)
         else:
             exc
-        ''')
+        ''', *expected)
 
     @skip('error reporting disabled due to false positives below')
     def test_undefinedExceptionNameObscuringGlobalVariable(self):
@@ -168,6 +173,8 @@ class Test(TestCase):
 
         Last line will never raise NameError because it's only entered
         if no exception was raised."""
+        # The exc variable is unused inside the exception handler.
+        expected = [] if version_info < (3,) else [m.UnusedVariable]
         self.flakes('''
         exc = 'Original value'
         def func():
@@ -178,13 +185,15 @@ class Test(TestCase):
                 print('exception logged')
                 raise
             exc
-        ''')
+        ''', *expected)
 
     def test_undefinedExceptionNameObscuringGlobalVariableFalsePositive2(self):
         """Exception names obscure globals, can't be used after. Unless.
 
         Last line will never raise NameError because `error` is only
         falsy if the `except:` block has not been entered."""
+        # The exc variable is unused inside the exception handler.
+        expected = [] if version_info < (3,) else [m.UnusedVariable]
         self.flakes('''
         exc = 'Original value'
         def func():
@@ -198,7 +207,7 @@ class Test(TestCase):
                 print(error)
             else:
                 exc
-        ''')
+        ''', *expected)
 
     def test_functionsNeedGlobalScope(self):
         self.flakes('''
@@ -719,14 +728,13 @@ class Test(TestCase):
             B = dict((i, str(i)) for i in T)
         ''')
 
-        if version_info >= (2, 7):
-            self.flakes('''
-            class A:
-                T = range(10)
+        self.flakes('''
+        class A:
+            T = range(10)
 
-                X = {x for x in T}
-                Y = {x:x for x in T}
-            ''')
+            X = {x for x in T}
+            Y = {x:x for x in T}
+        ''')
 
     def test_definedInClassNested(self):
         """Defined name for nested generator expressions in a class."""
@@ -752,7 +760,6 @@ class Test(TestCase):
         (42 for i in range(i))
         ''', m.UndefinedName)
 
-    @skipIf(version_info < (2, 7), 'Dictionary comprehensions do not exist')
     def test_definedFromLambdaInDictionaryComprehension(self):
         """
         Defined name referenced from a lambda function within a dict/set
@@ -771,7 +778,6 @@ class Test(TestCase):
         any(lambda: id(x) for x in range(10))
         ''')
 
-    @skipIf(version_info < (2, 7), 'Dictionary comprehensions do not exist')
     def test_undefinedFromLambdaInDictionaryComprehension(self):
         """
         Undefined name referenced from a lambda function within a dict/set
@@ -790,6 +796,24 @@ class Test(TestCase):
         any(lambda: id(y) for x in range(10))
         ''', m.UndefinedName)
 
+    def test_dunderClass(self):
+        """
+        `__class__` is defined in class scope under Python 3, but is not
+        in Python 2.
+        """
+        code = '''
+        class Test(object):
+            def __init__(self):
+                print(__class__.__name__)
+                self.x = 1
+
+        t = Test()
+        '''
+        if version_info < (3,):
+            self.flakes(code, m.UndefinedName)
+        else:
+            self.flakes(code)
+
 
 class NameTests(TestCase):
     """
